// Copyright 2014 by Jon Dart.  All Rights Reserved.

#ifndef _BOOK_IO_H
#define _BOOK_IO_H

#include "bookdefs.h"
#include "board.h"
#include "hash.h"

class BookIO {

        // provides access to the opening book.

        public:

        enum OpenMode {READ, WRITE, READ_WRITE};

        // opens book file.
        BookIO(const char* fileName, const OpenMode mode,
          bool create = false);
                
        // closes book file
        ~BookIO();
                
        bool is_open() const {
           return open;
        }
                
        // Fetches the list of book moves (up to "limit" moves) from the
        // file along with their associated scores.
        int book_moves(const Board &b, Move *moves, int *scores,  const unsigned
                       limit);

        // Randomly pick a move for board position "b" if available.
        Move book_move(const Board &b);

        void 

 protected:
        // Set "loc" to the location of the head of the hash chain
        // for the board position.
        void head(const Board &b, BookLocation &loc);

        void fetch(const BookLocation &loc, vector<DataEntry> &moves);
        
       void syncCurrentPage();
        
       int book_move_count(hash_t hashcode);
       
       protected:
               
       Move pickRandom(const Board &b, BookEntry * candidates,
          int * candidate_weights,
          int candidate_count,
          BookLocation *locs,BookInfo &info);

       int minWeight() const;

       FILE_HANDLE book_file;
       void *pBook;
       byte *pPage;
       MEM_HANDLE hFileMapping;
       bool open;
       int current_page;
       BookHeader hdr;
       
       private:
           
       void fetch_page(int page);
       int getBookMoves(const Board &b, const BookLocation &loc, BookEntry *moves, BookLocation *locs, int *scores, int limit);

};

#endif
